"""Assonant data classes types.

This submodule defines Enumations classes used to standardize type options existing in some data classes.
"""
from .scope_type import ScopeType
from .transformation_type import TransformationType

__all__ = [
    "ScopeType",
    "TransformationType",
]



"""Exceptions triggered to warn about internal errors in Assonant NeXus module."""


class NeXusObjectFactoryError(Exception):
    """Custom Exception to warn when any error occurs inside NeXusObjectFactory."""

    pass



"""Assonant DataHandler abstract class."""
from ..assonant_data_class import AssonantDataClass


# TODO: Make this class abstract
class DataHandler(AssonantDataClass):
    """Abstract class that creates the base common requirements to define an Assonant DataHandler."""



"""Assonant data classes - Subcomponents.

Data classes that defines available Assonant subcomponents.
"""
from .detector_module import DetectorModule
from .detector_roi import DetectorROI
from .monochromator_crystal import MonochromatorCrystal
from .monochromator_velocity_selector import MonochromatorVelocitySelector
from .subcomponent import Subcomponent

__all__ = [
    "DetectorROI",
    "DetectorModule",
    "MonochromatorCrystal",
    "MonochromatorVelocitySelector",
    "Subcomponent",
]



"""Assonant event base abstract class."""

from pydantic import BaseModel, ConfigDict


# TODO: make this class abstract
class AssonantEvent(BaseModel):
    """Assonant base event class.

    All Assonant event classes extends this class.
    """

    # Configuration dictionary used by pydantic (v2.0 >)
    model_config = ConfigDict(
        populate_by_name=True,  # Allows to populate with field name when an alias is given
        arbitrary_types_allowed=True,
        extra="forbid",  # https://github.com/pydantic/pydantic/issues/1887
    )



"""Assonant Monochromator Crystal subcomponent data class."""
from .subcomponent import Subcomponent


class MonochromatorCrystal(Subcomponent):
    """Data class to handle all data required to define a crystal from a Monochromator."""



"""Assonant Entry data class."""
from typing import Dict, List, Optional

from .assonant_data_class import AssonantDataClass
from .components import Component
from .data_handlers import DataHandler
from .types import ScopeType


class Entry(AssonantDataClass):
    """Data classes that wraps data into a logical/temporal scope related to the experiment.

    Entries are used to group and represent data in a defined temporal/logical scope of the
    experiment, which is directly define by the field "scope_type". e.g: calibration,
    pre-exposition.
    """

    scope_type: ScopeType
    components: Optional[List[Component]] = []
    fields: Optional[Dict[str, DataHandler]] = {}



"""Assonant TimeSeries class."""
from .data_field import DataField
from .data_handler import DataHandler


class TimeSeries(DataHandler):
    """Data class to handle any type of time series data."""

    value: DataField
    timestamps: DataField



"""Assonant File Writer.

Assonant File Writer module handle all process related to writing AssonantDataClasses data into
files in all supported file formats implemented.
"""

from .assonant_file_writer import AssonantFileWriter
from .exceptions import AssonantFileWriterError

__all__ = ["AssonantFileWriter", "AssonantFileWriterError"]



"""Assonant Data Classes.

Assonant data classes defines standard data schemas for acquiring sending,
manipulating and storing data over Assonant modules.

All classes inherits from AssonantDataClass and can be split in 3 types:

1. Instrument: Data classes responsible for grouping data related to instrumental data
2. Non-Instrument: Data classes responsible for grouping data related to non-instrumental data
3. Data Handler: Data classes responsible for standardizing how data is handled inside other
Assonant data classes.

To import and uses data classes from each type, refer to its specific submodule as shown below:

from .assonant_data_class.<sub_module_name> import <data_class_name>, ...
"""

from .assonant_data_class import AssonantDataClass
from .entry import Entry

__all__ = [
    "AssonantDataClass",
    "Entry",
]



"""Exceptions triggered to warn about internal errors in Assonant File Writer module."""


class AssonantFileWriterError(Exception):
    """Assonant Data Sender Error.

    Custom Exception to warn when any error occurs inside AssonantFileWriter
    """

    pass



"""Assonant Component abstract class."""
from typing import Dict, List, Optional

from ..assonant_data_class import AssonantDataClass
from ..data_handlers import Axis, DataHandler


# TODO: Make this class abstract
class Subcomponent(AssonantDataClass):
    """Abstract class that creates the base common requirements to define an Assonant Subcomponent.

    Subcomponents are less generic definitions which may compose and detail components. Usually,
    subcomponents existance doesn't make sense without being related to a Component.
    """

    name: str
    positions: Optional[List[Axis]] = []
    fields: Optional[Dict[str, DataHandler]] = {}



"""Assonant data classes - Components.

Data classes that defines available Assonant components.
"""

from .bvs import BVS
from .component import Component
from .detector import Detector
from .mirror import Mirror
from .monochromator import Monochromator
from .sample import Sample
from .slit import Slit

__all__ = [
    "Component",
    "Detector",
    "Mirror",
    "Monochromator",
    "Sample",
    "Slit",
    "BVS",
]



"""Assonant Detector Submodule subcomponent data class."""
from .subcomponent import Subcomponent


class DetectorModule(Subcomponent):
    """Data class to handle all data required to define a Module from a Detector."""



"""Assonant class to handle AssonantDataClasses transference over any communication method."""

from typing import Any, List

from ._assonant_data_sender_interface import IAssonantDataSender
from ._data_sender_factory import DataSenderFactory


class AssonantDataSender(IAssonantDataSender):
    """Assonant Data Sender. Wrapper class to deal with AssonantDataClass sending process.

    Wrapper class that abstracts all process related to creating specific data
    senders, sending data and response handling no matter the communication
    method used.
    """

    _factory = DataSenderFactory()

    def __init__(self, comm_method: str, configs: dict):
        # Persist comm_method and config informations on wrapper instance
        self.comm_method = comm_method
        self.configs = configs

        # Create specific data sender based on chosen comm_method
        self.data_sender = self._factory.create_data_sender(comm_method, configs)

    def send_data(self, payload: Any):
        """Method for sending data using the specific communication method implemented by the created Data Sender.

        Args:
            payload (Any): Any data that will be sent.
        """
        self.data_sender.send_data(payload)

    def get_supported_comm_methods(self) -> List[str]:
        """Getter method for current supported communication methods.

        Returns:
            List[str]: List containing curretly supported communication methods.
        """
        return self._factory.get_supported_comm_methods()



from abc import ABC, abstractmethod

from assonant.data_classes import AssonantDataClass


class IAssonantFileWriter(ABC):
    """Assonant File Writer Interface.

    This interface defines what must be implemented by a class to be a File Writer
    in Assonant environment
    """

    @abstractmethod
    def write_data(self, filepath: str, filename: str, data: AssonantDataClass):
        """Method resposible for writing data from AssonantDataClasses into a specific file format.

        Args:
            filepath (str): Path where file will be saved.
            filename (str): Name that will be given to file.
            data (AssonantDataClass): AssonantDataClass that contains data to be saved in file.
        """



"""Assonant Component abstract class."""
from typing import Dict, List, Optional

from ..assonant_data_class import AssonantDataClass
from ..data_handlers import Axis, DataHandler
from ..subcomponents import Subcomponent


# TODO: Make this class abstract
class Component(AssonantDataClass):
    """Abstract class that creates the base common requirements to define an Assonant Component.

    Components are more generic definitions which may be composed by many subcomponents if more detailing in its
    composition is desired.
    """

    name: str
    subcomponents: Optional[List[Subcomponent]] = []
    positions: Optional[List[Axis]] = []
    fields: Optional[Dict[str, DataHandler]] = {}



"""Assonant Detector ROI subcomponent data class."""
from .subcomponent import Subcomponent


class DetectorROI(Subcomponent):
    """Data class to handle all data required to define a Region of Interest(ROI) taken from a Detector."""



"""Assonant Monochromator Velocity Selector subcomponent data class."""
from .subcomponent import Subcomponent


class MonochromatorVelocitySelector(Subcomponent):
    """Data class to handle all data required to define a Velocity Selector from a Monochromator."""



"""Assonant NeXus.

Assonant toolkit for dealing with NeXus format.
"""

from .exceptions import NeXusObjectFactoryError
from .nexus_object_factory import NeXusObjectFactory

__all__ = ["NeXusObjectFactory", "NeXusObjectFactoryError"]
